package main

import (
    "bufio"
    "github.com/goshuirc/irc-go/ircmsg"
    "log"
    "os"
)

type Bot struct {
    Nick        string
    Ident       string
    Realname    string
    Server      Server
    ChansToJoin []string
    NSPasswd    string
    NSNick      string
    Logger      log.Logger
}

type BotConfig struct {
    Nick        string
    Ident       string
    Realname    string
    ChansToJoin []string
    NSNick      string
    NSPasswd    string
}

func newBot(config BotConfig, host, port, connectionName string, ssl bool) *Bot {
    logger := *log.New(os.Stdout, "", log.LstdFlags)

    return &Bot{
        Nick:        config.Nick,
        Ident:       config.Ident,
        Realname:    config.Realname,
        NSPasswd:    config.NSPasswd,
        NSNick:      config.NSNick,
        ChansToJoin: config.ChansToJoin,
        Server:      *NewServer(host, port, connectionName, ssl, logger),
        Logger:      logger,
    }
}

func (b *Bot) run() {
    err := b.Server.Connect()
    if err != nil {
        b.Logger.Fatal(err)
    }

    b.sendRaw(MakeMessage("NICK", b.Nick))
    b.sendRaw(MakeMessage("USER", b.Ident, "0", "0", b.Realname))
    b.readLoop()
}

func (b *Bot) sendRaw(message ircmsg.IrcMessage) {
    line, err := message.LineBytes()
    if err != nil {
        b.Logger.Printf("could not convert IrcMessage to string: %v", err)
        return
    }
    b.Logger.Printf(">> %s", line)
    b.Server.Conn.Write(line)
}

func (b *Bot) readLoop() {
    scanner := bufio.NewScanner(b.Server.Conn)
    for scanner.Scan() {
        b.HandleLine(scanner.Text())
    }
}

func (b *Bot) joinChannel(channelName string) {
    b.sendRaw(MakeMessage("JOIN", channelName))
}

func (b *Bot) sendPrivMessage(target, message string) {
    b.sendRaw(MakeMessage("PRIVMSG", message))
}
